package tools.descartes.dql.connector.kieker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import tools.descartes.dql.connector.AbstractOSGiQueryConnector;
import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.connector.QueryProgressListener;
import tools.descartes.dql.connector.kieker.structures.Trace;
import tools.descartes.dql.connector.kieker.structures.records.EventRecord;
import tools.descartes.dql.connector.kieker.structures.records.ResourceRecord;
import tools.descartes.dql.connector.kieker.util.KiekerHelper;
import tools.descartes.dql.connector.kieker.util.KiekerManager;
import tools.descartes.dql.core.engine.util.DQLLogger;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.StatType;
import tools.descartes.dql.models.mapping.domain.impl.DomainFactoryImpl;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class ModelStructureQueryConnectorImpl extends AbstractOSGiQueryConnector
		implements ModelStructureQueryConnector {
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private final DomainFactory domainFactory = DomainFactoryImpl.init();

	public ModelStructureQueryConnectorImpl() {
		super();
	}

	/**
	 * Creates and attaches probes to a list of probes
	 * 
	 * @param metrics
	 *            Metrics to be attached as probes
	 * @param probes
	 *            List of probes, where probes for metrics shall be attached
	 */
	private void attachProbes(final String[] metrics, final EList<Probe> probes) {
		Probe probe = null;

		for (final String metric : metrics) {
			probe = domainFactory.createProbe();
			probe.setMetricName(metric);

			probes.add(probe);
		}
	}

	@Override
	public String getBundleProperty(final String key) {
		return Activator.getContext().getProperty(key);
	}


	

	@Override
	public EntityMapping requestDoF(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = request.getModelLocation();
		final ArrayList<EObject> queryResults = new ArrayList<EObject>();

		return response;
	}

	@Override
	public EntityMapping requestEntities(final EntityMapping request) {

		final EntityMapping response = EcoreUtil.copy(request);
		
		KiekerManager manager = new KiekerManager(request);
		manager.run();
		final Trace queryResults = manager.getAnalysisResult();
		processListEntitiesResults(response,queryResults);

		return response;
	}

	@Override
	public EntityMapping requestProbes(final EntityMapping request) {
		
		final EntityMapping response = EcoreUtil.copy(request);
		
		processListMetricsResults(response);

		return response;
	}

	@Override
	public boolean supportsDoFRequests() {
		return true;
	}

	@Override
	public boolean supportsEntityRequests() {
		return true;
	}

	@Override
	public boolean supportsProbeRequests() {
		return true;
	}

	@Override
	public void addQueryProgressListener(QueryProgressListener arg0) {
		// TODO Auto-generated method stub

	}
	
	
	private void processListEntitiesResults(EntityMapping response, Trace kiekerResults) {
		List<ResourceRecord> resources = kiekerResults.getResources();
		List<EventRecord> events = kiekerResults.getEvents();
		
		final Map<String, String> resourceIdentifiers = new HashMap<>();
		final Map<String, String> serviceIdentifiers = new HashMap<>();
		
		 final EList<Entity> entities = response.getEntities();
		 for (ResourceRecord record: resources ) {
			 resourceIdentifiers.put(record.getIdentifier().getRessourceName(), "");
	 	 }
		 for (EventRecord record: events ) {
			 if (record.getIdentifier().getInterfaceID() != null && !record.getIdentifier().getInterfaceID().equals("")){
				 serviceIdentifiers.put(record.getIdentifier().getComponentID()+"."+record.getIdentifier().getInterfaceID(), "");
			 } else {
				 serviceIdentifiers.put(record.getIdentifier().getComponentID(), "");
			 }
	 	 }
		 
		 for (String identifier: resourceIdentifiers.keySet() ) {
			 Entity entity = DomainFactory.eINSTANCE.createEntity();
			 entity.setEntityType("RESOURCE");
			 entity.setIdentifier(identifier);
			 entities.add(entity);
	 	 }
		 for (String identifier: serviceIdentifiers.keySet() ) {
			 Entity entity = DomainFactory.eINSTANCE.createEntity();
			 entity.setEntityType("SERVICE");
			 entity.setIdentifier(identifier);
			 entities.add(entity);
	 	 }
	}
	
	private void processListMetricsResults(EntityMapping response) {

		for (Entity ent : response.getEntities()) {
			if  (ent.getEntityType().equals("RESOURCE")){
				Probe probe = DomainFactory.eINSTANCE.createProbe();
				probe.setMetricName(KiekerHelper.METRIC_UTILIZATION);
				StatType def = DomainFactory.eINSTANCE.createStatType();
				def.setTypeName("DEFAULT");
				StatType samp = DomainFactory.eINSTANCE.createStatType();
				samp.setTypeName("SAMPLE");
				probe.getStatTypes().add(def);
				probe.getStatTypes().add(samp);
				ent.getProbes().add(probe);
			} else  if (ent.getEntityType().equals("SERVICE")){
				Probe probe = DomainFactory.eINSTANCE.createProbe();
				probe.setMetricName(KiekerHelper.METRIC_RESPONSE_TIME);
				StatType def = DomainFactory.eINSTANCE.createStatType();
				def.setTypeName("DEFAULT");
				StatType samp = DomainFactory.eINSTANCE.createStatType();
				samp.setTypeName("SAMPLE");
				probe.getStatTypes().add(def);
				probe.getStatTypes().add(samp);
				ent.getProbes().add(probe);
			}
			
		}

	}
	
	

}
